import json, urllib.request, os
from difflib import SequenceMatcher

# This is the type of datasheet found in the json file
# Takes in a datasheet json object and
class Datasheet:
    date_created = None
    last_updated = None
    num_pages = None
    size_bytes = None
    date_created = None
    mimetype = None
    url = None
    source = None

    def __init__(self, datasheetobj):
        if datasheetobj is None: return
        if datasheetobj['metadata'] is not None:
            self.date_created = datasheetobj['metadata'].get('date_created')
            self.last_updated = datasheetobj['metadata'].get('last_updated')
            self.num_pages = datasheetobj['metadata'].get('num_pages')
            self.size_bytes = datasheetobj['metadata'].get('size_bytes')
            self.date_created = datasheetobj['metadata'].get('date_created')
        self.mimetype = datasheetobj['mimetype']
        self.url = datasheetobj['url']

        # print(datasheetobj)
        if datasheetobj['attribution']['sources'] is not None:
            self.source = datasheetobj['attribution']['sources'][0].get('name')

    def __respr__(self):
        return self.source

# This is the type of description object found in octopart json
class Description:
    value = None
    source = None

    def __init__(self, descriptionobj):
        if descriptionobj is None: return
        self.value = descriptionobj.get('value')
        self.source = descriptionobj['attribution']['sources'][0].get('name')

    def __respr__(self):
        return self.value

class Manufacturer:
    name = None
    homepage_url = None

    def __init__(self, manufacturerobj):
        if manufacturerobj is None: return
        self.name = manufacturerobj['name']
        self.homepage_url = manufacturerobj.get('homepage_url')

class PartOffer:
    in_stock_quantity = None

    def __init__(self, offerobj):
        if offerobj is None: return
        self.in_stock_quantity = offerobj.get('in_stock_quantity')

class SpecValue:
    display_value = None
    value = None
    max_value = None
    min_value = None

    def __init__(self, specobj):
        if specobj is None: return
        self.display_value = specobj.get('display_value')
        if specobj.get('value'): self.value = specobj.get('value')[0]
        self.max_value = specobj.get('max_value')
        self.min_value = specobj.get('min_value')

    def __respr__(self):
        return '{}: {}'.format(self.name, self.value)

class OctoCruncher:
    def __init__(self,
                 mpn=None,
                 file_source=None,
                 json_source=None
                 ):

        self.mpn = mpn
        self.file_source=file_source
        self.octoresp = json_source if json_source is not None else None
        self.itemnumber = 0
        self.api_key = os.environ.get('OCTOPART_API_KEY')

        self.__queryOctopart()

    # This does nothing now but will select which responded item you want
    def setItemNumber(self, itemnumber=0):
        self.itemnumber = itemnumber

    def getMPN(self):
        return self.octoresp['results'][0]['items'][self.itemnumber]['mpn']


    def getManufacturer(self):
        return Manufacturer(self.octoresp['results'][0]['items'][self.itemnumber]\
            ['manufacturer'])

    # Returns a Datasheet object
    def getDatasheet(self, datasheetnumber=0):
        return Datasheet(self.octoresp['results'][0]['items'][self.itemnumber]\
            ['datasheets'][datasheetnumber])

    # Returns a datasheet object
    def getDescription(self, descriptionnumber=0):
        return Description(self.octoresp['results'][0]['items'][self.itemnumber]\
            ['descriptions'][descriptionnumber])

    # This gets a parameter from the 'specs' dict of a response item
    def getSpec(self, specname):
        return SpecValue(self.octoresp['results'][0]['items'][self.itemnumber]\
            ['specs'].get(specname))


    # This does about the same thing but doesn't need an exact match
    def getSpecFuzzy(self, specname):
        # a dict that holds {spec_name, closeness_rating} e.g. {'resistance_tolerance', 9}
        howclose = {}

        for i in self.octoresp['results'][0]['items'][self.itemnumber]['specs']:
            if specname is None or i is None: continue
            howclose[i] = SequenceMatcher(None, specname, i).ratio()

        max = 0
        max_key = ''

        for i in howclose:
            if howclose[i] >= max:
                max = howclose[i]
                max_key = i

        return self.getSpec(max_key)

    # Get the number of datasheets
    def getNumDatasheets(self):
        return len(self.octoresp['results'][0]['items'][self.itemnumber]\
            ['datasheets'])

    # Get the number of descriptions
    def getNumDescriptions(self):
        return len(self.octoresp['results'][0]['items'][self.itemnumber]\
            ['descriptions'])

    def getJSON(self):
        return self.octoresp

    #Helper functions

    # Actually ask octopart for a response, or use the config file
    def __queryOctopart(self):
        if self.file_source is not None:
            with open(self.file_source) as f:
                self.octoresp = json.load(f)

            self.item = self.octoresp['results'][0]['items'][0]

            return

        print(self.mpn)
        print(self.api_key)
        # Build url for query
        url = 'http://octopart.com/api/v3/parts/match?'
        url += '&queries=[{"mpn":"' + self.mpn + '"}]'
        url += '&apikey=' + self.api_key
        url += '&include[]=datasheets&include[]=descriptions&include[]=specs'

        print(url)
        #
        # # Try to get the return from the cache
        # data = cache.get(cachekey)
        #
        # # If it doesn't exist (i.e. timed out or never run), run the query and cache it
        # if data is None:
        #     print("Querying API for {}".format(mpn))
        self.octoresp = json.loads(urllib.request.urlopen(url).read())
        self.item = self.octoresp['results'][0]['items'][0]

        return



# helpers.py
# All the messy functions that make octocruncher run
from difflib import SequenceMatcher

# Gives you a description from the list of them
def getDetail(specs, descriptions, type):
    type = type.lower()
    from octoconfig import description_types

    format_list = []

    if type in description_types:
        for i in range(len(description_types[type][1])):
            format_list.append(getSpecFuzzy(specs, description_types[type][1][i]))

        return description_types[type][0].format(*format_list)

    detail = ''

    # Primarily select from digikey
    for i in range(len(descriptions)):
        if 'digi' in descriptions[i]['attribution']['sources'][0]['name'].lower():
            return descriptions[i]['value']

    # If not available, go Mouser
    if detail == '':
        for i in range(len(descriptions)):
            if 'mouser' in descriptions[i]['attribution']['sources'][0]['name'].lower():
                return descriptions[i]['value']

    # If those fail, just suggest the first one
    if detail == '':
        return descriptions[0]['value']

    # If nothing else
    return ''

# Gets a nice simple value from a list of specs
def getValue(specs):
    from octoconfig import value_options

    return smash(specs, value_options)

# gets the package type
def getPackage(specs):
    return smash(specs, 'case_package')

# Mark critical if it can be replaced
def getCritical(type):
    from octoconfig import noncritical_types

    return '0' if type.lower() in noncritical_types else '1'

# This does a wild search
def getSpecFuzzy(specs, query):
    # a dict that holds {spec_name, closeness_rating} e.g. {'resistance_tolerance', 9}
    howclose = {}
    for i in specs:
        howclose[i] = SequenceMatcher(None, query, i).ratio()

    max = 0
    max_key = ''

    for i in howclose:
        if howclose[i] >= max:
            max = howclose[i]
            max_key = i

    return smash(specs, max_key)

    # return max from howclose dict



def getLibRef(specs, type):
    if type == "Resistor": return 'Resistor - Standard'
    if type == "Capacitor": return 'Capacitor - Bipolar'
    if type == "Inductor": return 'Inductor - Core'
    return ''


def getFootprintRef(package, type):
    if type == "Resistor": return 'R{}L'.format(package)
    if type == "Capacitor": return 'C{}L'.format(package)
    if type == "Inductor": return 'L{}L'.format(package)
    return ''



# This function takes a dictionary and finds an allowed value if it exists
def smash(mydict, allowable_values):
    # If only a single string, no need to iterate
    if isinstance(allowable_values, str):
        return getvalue(allowable_values.lower(), mydict)

    # Run some tests for each acceptable value
    for i in allowable_values:
        x = getvalue(i.lower(), mydict)
        if x != '': return x

    return ''

# Quick function to get display_val if avalable
def getvalue(i, mydict):
    if i in mydict:
        if 'display_value' in mydict[i]:
            return mydict[i]['display_value']
        if 'value' in mydict[i]:
            return mydict[i]['value']
        if 'min_value' in mydict[i] and 'max_value' in mydict[i]:
            return mydict[i]['min_value'] + '-' + mydict[i]['max_value']
    return ''
